# Library Aliquots

A library aliquot is a portion of library, or an entire library, which may or may not have been diluted in preparation
for pooling or adding to a sequencing container. Only library aliquots can be added to pools in MISO, and only pools
can be added to sequencing containers. This means that you will need to create a library aliquot in MISO even if the
library is being pooled as is, or if it is being added directly to a sequencing container.

## Naming Scheme

MISO can be configured with a naming scheme appropriate for your organization. For more details about this
configuration, see [Naming Scheme](../site_configuration/#naming-schemes) in the Site Configuration section.

For library aliquots, the naming scheme specifies whether or not duplicate aliases are allowed, and may also specify a
pattern that all library aliquot aliases must follow. If such a pattern is specified, then the aliases will be compared
to this pattern whenever saving a library aliquot. If the alias does not follow the pattern specified by the naming
scheme, or if there is a duplicate alias when not allowed, the save will fail and a validation message will state the
cause.

The naming scheme may also specify a library aliquot alias generator. This means that users will not have to choose
library aliquot aliases because they will be generated automatically when saving library aliquots. The user is still
free to choose a library aliquot alias by typing it in manually if they would like.

## Library Aliquots List

To get to the main Library Aliquots list page, click "Library Aliquots" in the Preparation list in the menu on the left
side of the screen. This list includes all library aliquots from all projects in MISO. The toolbar at the top of the
table includes many controls for working with library aliquots.

You can find a similar list in the Library Aliquots section of the Edit Project page. The list on the Edit Project page
only includes the library aliquots belonging to the project being viewed/edited. This list has the same controls as on
the main Library Aliquots list page.

Clicking on a library aliquot's name will take you to the Edit Library Aliquot page. Clicking on a library name or
alias will take you to the Edit Library page.



## Propagating Library Aliquots from Libraries

To make library aliquots from libraries, go to the Libraries list, check the checkboxes next to the libraries you wish
to propagate from, then click the "Make aliquots" button in the toolbar at the top of the table. A dialog will appear
and give you the option to create a box for these new library aliquots. See [Creating Boxes](../boxes/#creating-boxes)
in the box section more more information on this feature. When you are done in the dialog, click the "Create" button to
proceed.

This will take you to the bulk Create Library Aliquots from Libraries page. Enter all of the aliquot information and
then click the "Save" button at the top right of the screen when you are done. For additional information regarding
some of the fields in the table, check the Quick Help section near the top of the screen. If any of the library
aliquots fail to save, a message will be displayed at the top of the screen explaining the problem(s). You can then
make the appropriate adjustments and click the "Save" button to try again.



## Propagating Library Aliquots from Library Aliquots

Making library aliquots from library aliquots works exactly like making them from libraries except that you start by
selecting library aliquots to propagate from on the Library Aliquots list and clicking the "Propagate" button at the
top of the table.



## Editing Library Aliquots

Library aliquots can be edited individually or in bulk. Bulk editing is convenient when you have several library
aliquots to modify at once. Editing individual library aliquots is sometimes preferable because it provides a few
additional options, and allows you to see a more detailed view of the library aliquot.



### Editing a Single Library Aliquot

To get to the single Edit Library Aliquot page, click on the aliquot's name in the Library Aliquots list. Links to the
Edit Library Aliquot page also appear on the Edit Pool page in the Included and Available Library Aliquots lists.

The Library Aliquot Information section of the Edit Library Aliquot page contains a list of fields, most of which may
be modified. You can make any changes you would like and then click the "Save" button at the top right to confirm the
changes.

Other sections below list the pools that the library aliquot has been added to, the runs that those pools have been
loaded into, and the aliquot's change log.



### Editing Library Aliquots in Bulk

To get to the bulk Edit Library Aliquots page, go to the Library Aliquots list, check the checkboxes beside the
aliquots you wish to modify, then click the "Edit" button in the toolbar at the top of the table.

The bulk Edit Library Aliquots page works similarly to the bulk Create Library Aliquots from Libraries page. You can
make the changes you would like, then click the "Save" button at the top right to confirm. If there are any problems
with the data you've entered, the error messages will be displayed at the top of the screen and you can adjust as
necessary before clicking "Save" again.



## Pooling Library Aliquots Together

To create a single new pool containing one or several library aliquots, go to the Library Aliquots list, select the
aliquots that you would like to include in the pool, and click the "Pool together" button in the toolbar at the top of
the table. Further details of this process will be discussed in
[Creating Pools from Library Aliquots](../pools/#creating-pools-from-library-aliquots).



## Pooling Library Aliquots Separately

To create multiple new pools, each containing a single library aliquot, go to the Library Aliquots list, select the
aliquots that you would like to create pools for, and click the "Pool separately" button in the toolbar at the top of
the table. Further details of this process will be discussed in
[Creating Pools from Library Aliquots](../pools/#creating-pools-from-library-aliquots).



## Custom Pooling in Bulk

To create multiple new pools, each containing a variable number of library aliquots, go to the Library Aliquots list,
select all of the aliquots that you would like to include in any of the pools, and click the "Pool custom" button in
the toolbar at the top of the table. Further details of this process will be discussed in
[Creating Pools from Library Aliquots](../pools/#creating-pools-from-library-aliquots).



## Printing Barcodes

Barcode labels can be printed for a series of library aliquots from the Library Aliquots list. See the
[Barcode Label Printers section - Printing Barcodes](../barcode_label_printers/#printing-barcodes) for details on how
to do this.



## Downloading Library Aliquot Information

It is sometimes useful to have library aliquot information in spreadsheet form for use outside of MISO. To download
library aliquot data, go to the Library Aliquots list, check the checkboxes next to the aliquots that you would like to
export, and click the "Download" button in the toolbar at the top of the table. A dialog will appear in which you can
choose the type and format of the spreadsheet before downloading. Different spreadsheet types include different
columns. Feel free to try out the different types to see which ones will be useful to you. MISO supports exporting
spreadsheets in Microsoft Excel (xlsx), Open Document (odt), and comma-delimited (csv) formats.



## Selecting Library Aliquots by Search

If you have a list of library aliquot names or barcodes, or library names, aliases, or barcodes, you can use this list
to select and act upon the aliquots instead of having to search for and select them manually. For more information on
this feature, see the [General Navigation section - Selecting by Search](../general_navigation/#selecting-by-search).



## Finding Related Items

You can find items related to selected library aliquots using the "Parents" and "Children" buttons in the toolbar at
the top of the Library Aliquots list. For more information on this feature, see the
[General Navigation section - Finding Related Items](../general_navigation/#finding-related-items).



## Adding Library Aliquots to a Workset

It can sometimes be difficult to find and select a group of library aliquots that you want to work with. To make this
easier, you can create a workset that includes all of the aliquots that you want grouped together. For more information
on this feature, see the [Worksets section](../worksets/).



## Deleting Library Aliquots

To delete library aliquots, go to the Library Aliquots list, check the checkboxes next to the aliquots that you wish to
delete, and click the "Delete" button in the toolbar at the top of the table. A library aliquot can only be deleted if
it has not been added to any pools. If the aliquot has been added to any pools, either these pools must be deleted, or
the aliquot must be removed from them before the aliquot can be deleted. An aliquot can only be deleted by its creator
or a MISO administrator.

