var Transfer = (function($) {

  var itemsListId = 'listItems';

  var form = null;
  var itemsListConfig = {};

  return {

    setForm: function(formApi) {
      form = formApi;
    },

    setItemsListConfig: function(config) {
      itemsListConfig = config;
    },

    setItems: function(items) {
      FormUtils.setTableData(ListTarget.transferitem, itemsListConfig, itemsListId, items, form);
      updateBulkActions(items);
    },

    getItems: function() {
      return FormUtils.getTableData(itemsListId);
    },

    addItems: function(addItems) {
      var items = Transfer.getItems().concat(addItems);
      Transfer.setItems(items);
    },

    removeItems: function(removeItems) {
      var items = removeByTypeAndId(Transfer.getItems(), removeItems);
      Transfer.setItems(items);
    },

    updateItems: function(updatedItems) {
      var items = removeByTypeAndId(Transfer.getItems(), updatedItems).concat(updatedItems);
      Transfer.setItems(items);
    }

  };

  function removeByTypeAndId(allItems, removeItems) {
    return allItems.filter(function(transferItem) {
      return !removeItems.some(function(removeItem) {
        return removeItem.type === transferItem.type && removeItem.id === transferItem.id;
      });
    });
  }

  function updateBulkActions(items) {
    if (!items || !items.length) {
      setNoBulkActionsMessage('Add items to transfer to see bulk actions.');
      return;
    }

    var itemTypes = Utils.array.deduplicateString(items.map(function(item) {
      return item.type;
    }));

    if (itemTypes.length > 1) {
      setNoBulkActionsMessage('Bulk actions not available for multi-type transfers.');
      return;
    }

    switch (itemTypes[0]) {
    case 'Sample':
      setBulkActions(HotTarget.sample.getBulkActions({}));
      break;
    case 'Library':
      setBulkActions(HotTarget.library.getBulkActions({}));
      break;
    case 'Library Aliquot':
      setBulkActions(HotTarget.libraryaliquot.getBulkActions({}));
      break;
    case 'Pool':
      setBulkActions(HotTarget.pool.getBulkActions({}));
      break;
    default:
      throw new Error('Unexpected item type: ' + itemTypes[0]);
    }
  }

  function setNoBulkActionsMessage(message) {
    setToolbar($('<span>').text(message));
  }

  function setBulkActions(actions) {
    var getItems = function() {
      return FormUtils.getSelectedTableData(itemsListId);
    }

    setToolbar(actions.filter(function(action) {
      return !!action;
    }).map(function(action) {
      return Utils.ui.makeBulkActionButton(action, getItems);
    }));
  }

  function setToolbar(items) {
    $('#type-toolbar').remove();
    var toolbar = $('<div>').prop('id', 'type-toolbar');
    toolbar.append(items);
    $('#listItems > .fg-toolbar').append(toolbar);
  }

})(jQuery);